<?php

require_once 'Minify/Controller/Base.php';

/**
 * Controller class for serving a single HTML page
 * 
 * @link http://code.google.com/p/minify/source/browse/trunk/web/examples/1/index.php#59
 * 
 */
class Minify_Controller_Page extends Minify_Controller_Base {
    
    /**
     * Set up source of HTML content
     * 
     * @param array $options controller and Minify options
     * @return array Minify options
     * 
     * Controller options:
     * 
     * 'content': (required) HTML markup
     * 
     * 'id': (required) id of page (string for use in server-side caching)
     * 
     * 'lastModifiedTime': timestamp of when this content changed. This
     * is recommended to allow both server and client-side caching.
     * 
     * 'minifyAll': should all CSS and Javascript blocks be individually 
     * minified? (default false) 
     */
    public function setupSources($options) {
        // strip controller options
        $sourceSpec = array(
            'content' => $options['content']
            ,'id' => $options['id']
        );
        unset($options['content'], $options['id']);
        
        if (isset($options['minifyAll'])) {
            // this will be the 2nd argument passed to Minify_HTML::minify()
            $sourceSpec['minifyOptions'] = array(
                'cssMinifier' => array('Minify_CSS', 'minify')
                ,'jsMinifier' => array('Minify_Javascript', 'minify')
            );
            $this->_loadCssJsMinifiers = true;
            unset($options['minifyAll']);
        }
        $this->sources[] = new Minify_Source($sourceSpec);
        
        // may not be needed
        //$options['minifier'] = array('Minify_HTML', 'minify');
        
        $options['contentType'] = Minify::TYPE_HTML;
        return $options;
    }
    
    private $_loadCssJsMinifiers = false;
    
    /**
     * @see Minify_Controller_Base::loadMinifier()
     */
    public function loadMinifier($minifierCallback)
    {
        if ($this->_loadCssJsMinifiers) {
            // Minify will not call for these so we must manually load
            // them when Minify/HTML.php is called for.
            require 'Minify/CSS.php';
            require 'Minify/Javascript.php';
        }
        parent::loadMinifier($minifierCallback); // load Minify/HTML.php
    }
}

